package com.yeejoin.amos.iec104.tcp.client.invoke;

import java.util.LinkedList;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.Executors;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import com.yeejoin.amos.connect.dao.entity.PointConfigVo;
import org.apache.log4j.Logger;
import org.springframework.util.ObjectUtils;

import com.yeejoin.amos.iec104.business.constants.IEC104Constant;
import com.yeejoin.amos.iec104.business.param.AlarmParam;
import com.yeejoin.amos.iec104.business.service.intfc.IClientListenService;
import com.yeejoin.amos.iec104.business.service.intfc.IConfService;
import com.yeejoin.amos.iec104.business.service.intfc.IPointConfigService;
import com.yeejoin.amos.iec104.context.FireAutoIntfContext;

public class GeneralDataQueue {
	private static final BlockingQueue<AlarmParam> blockingQueue = new LinkedBlockingQueue<AlarmParam>();
	ScheduledExecutorService service_general = null;
	IClientListenService clientListenService;
	IPointConfigService pointConfigService;
	IConfService confService;
	private volatile static GeneralDataQueue instance = null;
	
	private GeneralDataQueue() {
		clientListenService = (IClientListenService) FireAutoIntfContext.getInstance().getBean(IClientListenService.class);
		confService = (IConfService) FireAutoIntfContext.getInstance().getBean(IConfService.class);
		pointConfigService = (IPointConfigService) FireAutoIntfContext.getInstance().getBean(IPointConfigService.class);
	}
	
    public static GeneralDataQueue getInstance(){
    	//先检查实例是否存在，如果不存在才进入下面的同步块
         if(instance == null){
        	 //同步块，线程安全的创建实例
             synchronized(GeneralDataQueue.class){
            	 //再次检查实例是否存在，如果不存在才真的创建实例
                 if(instance == null){
                     instance = new GeneralDataQueue();
                 }
             }
         }
         return instance;
    }
	
    /**
     * 启动apdu数据消费任务
     */
    public void start() {
    	service_general = Executors.newSingleThreadScheduledExecutor();
        // 第二个参数为首次执行的延时时间，第三个参数为定时执行的间隔时间
    	service_general.scheduleAtFixedRate(task_runnable, 5, 10, TimeUnit.SECONDS);
    }
    
	public void add(AlarmParam recData) {
		blockingQueue.add(recData);
	}
	
	Runnable task_runnable = new Runnable() {
        public void run() {
            while(true) {
            	try {
            		AlarmParam param1 = blockingQueue.take();
            		LinkedList<AlarmParam> datas = new LinkedList<AlarmParam>();
            		PointConfigVo vo = pointConfigService.findByServicesIdAndInfoAddress(param1.getServiceId(), Integer.valueOf(param1.getInformationAddress()));
            		param1.setPointCode(vo.getPointCode());
            		datas.add(param1);
            		boolean isEmpty = false;
            		do {
            			AlarmParam param = blockingQueue.poll(1, TimeUnit.SECONDS);
            			if (!ObjectUtils.isEmpty(param)) {
            				PointConfigVo vo1 = pointConfigService.findByServicesIdAndInfoAddress(param.getServiceId(), Integer.valueOf(param.getInformationAddress()));
            				if (!ObjectUtils.isEmpty(vo1)) {
            					
            					param.setPointCode(vo1.getPointCode());
                				datas.add(param);
                				if (datas.size() > 10000) {
                					clientListenService.sendDatas(IEC104Constant.GENERAL_MESSAGE, datas);
                					datas.clear();
                				}
            				} else {
            					Logger.getLogger(this.getClass()).error(param.getServiceId() + " have not " + param.getInformationAddress());
            				}
            			} else {
            				isEmpty = true;
            			}
            		} while(!isEmpty);
            		try {
            			if (!ObjectUtils.isEmpty(datas)) {
            				clientListenService.sendDatas(IEC104Constant.GENERAL_MESSAGE, datas);
            			}
            			datas.clear();
            		} catch (Exception e) {
            		}
				} catch (Exception  e) {
					Logger.getLogger(this.getClass()).error(e.getMessage());
					e.printStackTrace();
				} 
            }
        }
    };
}
